mod Mkleene

incl Mtex

incl Mtexrc

incl Mtexgr

incl Maux

incl Mterm

incl Mrd

rem 
  \para \bf  Kleene's T-predicate.  \end 

fun U 
  U(0) = 0
  U(k,z) = z

pred/3 T1 
  T1(e,x1,y) <-> 
    Cdf(1,e) & 
    \eb k k <= y: \eb z z <= y: (y = k,z & Ird(k,At(e,Cnm(x1))) = Cnm(z))

pred/4 T2 
  T2(e,x1,x2,y) <-> 
    Cdf(2,e) & 
    \eb k k <= y: \eb z z <= y: (y = k,z & Ird(k,At(e,Cnm(x1),Cnm(x2))) = Cnm(z))

pred/5 T3 
  T3(e,x1,x2,x3,y) <-> 
    Cdf(3,e) & 
    \eb k k <= y: \eb z z <= y: (y = k,z & 
                                 Ird(k,At(e,Cnm(x1),Cnm(x2),Cnm(x3))) = Cnm(z))

rem 
  \para \bf  Graph of the unary partial enumeration function.  \end 

pred/3 Graph 'Tex_p3_comp_graph_enum'
  Graph(e,x,y) <-> \e kT1(e,x,k,y)

rem 
  \para \bf  Indices of identity functions.  \end 

fun/2 Xi 'Tex_f2_gr_xi'
  Xi(n,i) = Ls(n,Vt(i))

rem 
  \para \bf  Indices of constant functions.  \end 

fun/2 Xc 'Tex_f2_gr_xc'
  Xc(n,m) = Ls(n,Cnm(m))

rem 
  \para \bf  Indices of nowhere defined partial functions.  \end 

fun/2 Vars 
  Vars(i,n) = Vt(n) <- i >= n
  Vars(i,n) = Vt(i),Vars(i+1,n) <- i < n

fun Xundef 'Tex_f1_gr_xundef'
  Xundef(n) = Ls(n,At(Fs(n),Vars(1,n)))

rem 
  \para \bf  Turning recursive indices into well-formed ones.  \end 

fun/2 Xwf 'Tex_f2_gr_xwf'
  Xwf(n,e) = e <- Cdf(n,e)
  Xwf(n,e) = Xundef(n) <- ~Cdf(n,e)

rem 
  \para \bf  Unary composition.  \end 

fun/2 Xcompu 'Tex_f2_gr_xcompu'
  Xcompu(e1,e2) = Ls(1,At(Xwf(1,e1),At(Xwf(1,e2),Vt(1))))

rem 
  \para \bf  Composition.  \end 

fun/3 Xcomp1 
  Xcomp1(n,1,e) = At(Xwf(n,e),Vars(1,n))
  Xcomp1(n,m+2,e,es) = At(Xwf(n,e),Vars(1,n)),Xcomp1(n,m+1,es)

fun/4 Xcomp 'Tex_f4_gr_xcomp'
  Xcomp(n,m,e,es) = Ls(n,At(Xwf(m,e),Xcomp1(n,m,es)))

rem 
  \para \bf  Parametric function.  \end 

fun/2 Xparam 'Tex_f2_gr_xparam'
  Xparam(e,x) = Xcomp(1,2,e,Xc(1,x),Xi(1,1))

rem 
  \para \bf  S-m-n theorem.  \end 

fun/3 Xsmn1 
  Xsmn1(1,n,x) = Xc(n,x)
  Xsmn1(m+2,n,x) = Xc(n,x),Xsmn1(m+1,n,x)

fun/2 Xsmn2 
  Xsmn2(i,n) = Xi(n,n) <- i = n
  Xsmn2(i,n) = Xi(n,i),Xsmn2(i+1,n) <- i < n

fun/4 Xsmn 'Tex_f4_gr_xsmn'
  Xsmn(m,n,e,xs) = Xcomp(n,m+n,e,Conc(m,Xsmn1(m,n,xs),Xsmn2(1,n)))

